;
;		TRK_202B.ASM
;               Copywrite N0QBH   2007
;		PIC 16F628A GPS to APRS
;		Software/DAC tone generation
;               Programable callsign & path stored in EEPROM memory
;		All GPS data stored in mid memory
;		Tail text every 8 transmissions
;		User select standard or mic-e format
;		Adjusted sine table w/voltage centering
;		New EEPROM config storage routine
;		Status bits B,C are configurable on the fly
;		
;               3/25/07
;               10 Mhz CLOCK	Ver 2.02B
;------------------------------------------------------------
;
        LIST    P=16F628A
	ERRORLEVEL	-302
        include <P16F628A.INC>
        __CONFIG        h'3d22'
;-------------------------------------------------------------
;               Port Set-up
;           0   1   2   3   4   5   6   7       bit
; port  A   Out Out In  Out In  X   X   X	
;       B   Out In  In  Out In  Out In  In	
;------------------------------------------------------------
	ORG	0			; start code
	goto	main
	ORG	4
	goto	int_service		;interrupt vector
;------------------------------------------------------------
;
        cblock  0x20
                temp_S                  ; 20
                temp_W                  ; 21 int regs
		temp_PCLH		; 22
;
                temp                    ; 23 buffer
                phase                   ; 24 DAC buffer
                CFLAGS                  ; 25 mic-e compressed flags
                flag_count              ; 26 flag counter
                send_reg                ; 27 data byte
                bit_count               ; 28 data bit cntr
                bit_stuff_count         ; 29 bit stuffing
                AX25_crc_hi             ; 2a for CRC
                AX25_crc_lo             ; 2b for CRC
                next_freq               ; 2c 0/1 tone / cfg routine 
                lookup_count            ; 2d ROM table offset
                loop_count              ; 2e loop down counter
		sine_offset 		; 2f sine lookup offset
                rx_serial               ; 30 serial data
                FLAGS                   ; 31 various flag bits
		buffer_1		; 32 beacon timer
		tx_serial		; 33 tx serial
		offset			; 34
		ee_addr			; 35
		ee_data			; 36
		ascii_10s		; 37 ascii to hex
		ascii_1s		; 38
		hex_result		; 39
		count			; 3a counter (not cleared)
		beacon_count		; 3b holdoff (not cleared)
		mice_yn			; 3c mic-e switch (not cleared)
		m_symbol		; 3d mic-e symbol (not cleared)
        endc
;
	cblock 0x40
		lat_d10
		lat_d1
		lat_m10
		lat_m1
		lat_m0
		lat_m01
		lon_d
		lon_m0
		lon_m00
		spd_100
		spd_10
		spd_1
		crs_100
		crs_10
		crs_1			; 4e
	endc
;
;------------------------------------------------------------
#define TONE            FLAGS,0         ;tone freq hi/lo
#define START_SERIAL    FLAGS,1         ;serial routine begin
#define	NEW_DATA	FLAGS,2		;async byte rx'd
#define BUSY_FLAG	FLAGS,3		;channel activity flag
#define	ERR_FLAG	FLAGS,4		;user input too long
#define	AH_FLAG		FLAGS,5		;ascii hex error
#define	BT_FLAG		FLAGS,6		;<CR> w/no data
;
#define NS_FLAG		CFLAGS,0	;north or south latitude
#define	EW_FLAG		CFLAGS,1	;east or west longitude
#define	LD09_FLAG	CFLAGS,2	;longitude (0-9)(100-109)
#define	LD100_FLAG	CFLAGS,3	;longitude > 99
#define	LM09_FLAG	CFLAGS,4	;longitude < 10 minutes	
;
#define TOGGLE          PORTA,4         ;load/read eemem
#define PTT_OUT         PORTA,1         ;PTT when set
#define LED             PORTB,0         ;LED hi
;
LO_TONE		equ	d'130'		;1200 Hz(52uS x 16)
;
HI_TONE		equ	d'71'		;2200 Hz(28.4uS x 16)
;
BIT_T		equ	d'63454'	;832.8 uS to overflow
;
RAML_START	equ	0x50		;low RAM start address
RAML_END	equ	0x7f		;low RAM end 
RAMM_START	equ	0xa0		;mid RAM start address
RAMM_END	equ	0xef		;mid RAM end
BANK2_START	equ	0x20		;callsign loc in bank 2
BANK2_END	equ	0x4f		;last byte bank 2
SCRATCH_PAD	equ	0x70		;temp data loc
HOLDOFF		equ	0x30		;txd loc in eemem
MICE_SW		equ	0x31		;mic-e switch in eemem
MICE_SYMBOL	equ	0x32		;mic-e symbol in eemem
CR		equ	0x0d		;carriage return
LF		equ	0x0a		;linefeed
BACKSPACE	equ	0x08		;backspace
;
;------------------------------------------------------------
;	
data_table
        addwf   PCL,F                   ;add offset to pc
        dt      "GPRMC,,A,*xx",0 
;
;------------------------------------------------------------
;	Text between "quotes" can be edited	
tail_text
        addwf   PCL,F                   ;add offset to pc
	dt	" FreeTrak",0
;
;------------------------------------------------------------
main
        movlw   0x07
        movwf   CMCON			;analog inputs off
        bsf     STATUS,RP0              ;switch to bank 1
        movlw   b'00010100'             ;port A in/outputs
        movwf   TRISA
        movlw   b'11010110'             ;port B in/outputs
        movwf   TRISB
        movlw   b'10010000'             ;INTF hi/lo, TMR0/2
        movwf   OPTION_REG		;
        movlw   b'00100010'             ;RCIE, TMR2IE on
        movwf   PIE1                    ;perif int setup
        movlw   b'11000000'
        movwf   VRCON			;Vref On, RA2, Hi Range 
        movlw   b'00000000'
        movwf   TXSTA                   ;tx off, BRGH lo
        movlw   d'32'			;
        movwf   SPBRG                   ;4K8 data speed
	movlw	LO_TONE
	movwf	PR2			;52 uS timeout
        bcf     STATUS,RP0              ;back to page 0
;
        movlw   b'10010000'             ;serial rx enabled
        movwf   RCSTA                   ;
        clrf    PIR1                    ;reset perif int
        movlw   b'01000000'             ;PIE1 on
	movwf	INTCON
        movlw   b'00000001'		;
        movwf   T1CON                   ;TMR1 /1, on
	movlw	b'00000000'		;
	movwf	T2CON			;TMR2 post/1, off 
        bcf     PTT_OUT			;PTT off
        bcf     LED                     ;LED off
;
;------------------------------------------------------------
;	Selects either CFG or RUN mode
;
check_tog_set
        btfss   TOGGLE                  ;load preference
	goto	tocall_area		;get path input serially
	call	load_stored_path	;run with stored eedata
	bsf	INTCON,GIE		;enable interrupts
	movlw	1
	movwf	count			;tail msg on 1st tx
;------------------------------------------------------------
;               Clear registers
clean_house
        movlw   0x22
        movwf   FSR
clean_loop
        clrf    INDF
        incf    FSR,W
        xorlw   0x3a			;don't clear beacon_count,
        bz      load_holdoff		;count and mice_yn
        incf    FSR,F
        goto    clean_loop
;------------------------------------------------------------
load_holdoff
	movfw	beacon_count
	movwf	buffer_1		;load holdoff
;
begin_parse
	bcf	LED
	bsf	START_SERIAL		;start at begining
;
serial_rx
	btfss	NEW_DATA		;new byte rx'd?
	goto	serial_rx		;no
	bcf	NEW_DATA		;yes, clear flag
;
	btfss	START_SERIAL		;are we at beginning?
	goto	serial_running		;no
	clrf	lookup_count		;yes, reset pointers
	movlw	RAMM_START
	movwf	FSR			;reset FSR
;
;============================================================
serial_running
	movfw	lookup_count		;load pointer
	call	data_table		;point to ROM data
	movwf	temp
	movfw	temp
	bz	compress_option		;seen the end, do next
	xorlw	"x"
	bz	store_varible
;
	movfw	temp
	xorlw	"*"			;store til star is seen?
	bz	store_til_CS		;yes
;
	movfw	temp
	xorlw	","
	bz	store_til_comma
;
	movfw	temp			;
	xorwf	rx_serial,W		;no, match ROM string ?
	bz	lookup_match		;yes
;
lookup_restart
	bsf	START_SERIAL		;no match, start anew
	goto	serial_rx		;look for next
;
lookup_match
	bcf	START_SERIAL		;match, clear flag, continue
;
store_varible
	incf	lookup_count,F
	goto	store_rxd_byte		;look for next
;
store_til_comma
	movlw	","
	xorwf	rx_serial,W
	skpnz
	incf	lookup_count,F		;incf lookup if , is seen
	goto	store_rxd_byte
;
store_til_CS
	movlw	"*"
	xorwf	rx_serial,W
	skpnz	
	incf	lookup_count,F		;incf lookup if * is seen
;
store_rxd_byte
	movfw	rx_serial		;
	movwf	INDF			;store rx'd byte
	movlw	RAMM_END
	xorwf	FSR,W			;at the low RAM end?
	bz	clean_house		;yes, overrun-restart
	incf	FSR,F			;no, inc store offset
	goto	serial_rx		;look for next
;
compress_option
	btfss	mice_yn,0
	goto	ready_to_tx		;stnd format
;
;============================================================
;
;	We start the sorting for compression here
;
mic_e_compressor
	movlw	"0"
	movwf	spd_100			;preload with ascii 0
	movwf	spd_10
	movwf	spd_1
	movwf	crs_100
	movwf	crs_10
	movwf	crs_1
;
	movlw	RAMM_START
	movwf	offset			;FSR to offset
	call	findacomma		;go til 1st comma seen
	incf	offset,F
	call	findacomma		;go til 2nd comma seen
	incf	offset,F
	call	findacomma		;go til 3rd comma seen
;
	incf	FSR,F			;point to start of data
	movfw	INDF
	movwf	lat_d10			;lat deg 10's
	incf	FSR,F			;to next data
	movfw	INDF
	movwf	lat_d1			;lat deg 1's
	incf	FSR,F			;to next data
	movfw	INDF
	movwf	lat_m10			;lat min 10's
	incf	FSR,F			;to next data
	movfw	INDF
	movwf	lat_m1			;lat min '1s
;
	incf	FSR,F			
	movlw	"."
	xorwf	INDF,W			;test for decimal pt
	bnz	clean_house		;error, reset
;	
	incf	FSR,F			;to next data	
	movfw	INDF
	movwf	lat_m0			;lat min 10ths
	incf	FSR,F			;to next data
	movfw	INDF
	movwf	lat_m01			;lat min 100ths
	movfw	FSR
	movwf	offset			;FSR to offset
	call	findacomma		;go til comma seen
	incf	FSR,F			;to N S data
;
	movlw	"N"
	xorwf	INDF,W
	bnz	test_for_south
	bsf	NS_FLAG			;if N set flag
	goto	dothe_longitude
test_for_south
	movlw	"S"
	xorwf	INDF,W
	bnz	clean_house		;error, reset
	bcf	NS_FLAG			;if S clear flag
;
;
dothe_longitude
	bcf	LD100_FLAG		;if set long deg >99
	bcf	LD09_FLAG		;if set long deg (0-9 100-109)
	bcf	LM09_FLAG		;if set long min <10
	incf	FSR,F			;skip comma
	incf	FSR,F			;to next data
;
	movlw	"1"
	xorwf	INDF,W
	skpnz
	bsf	LD100_FLAG		;longitude 100-179
	incf	FSR,F			;to longitude deg 10's
	movlw	"0"
	xorwf	INDF,W
	skpnz
	bsf	LD09_FLAG		;longitude 0-9	
	movfw	INDF
	movwf	ascii_10s
	incf	FSR,F			;to longitude deg 1's
	movfw	INDF
	movwf	ascii_1s
	call	ascii_to_hex
	movfw	hex_result
	movwf	lon_d			;store as one hex byte
;
	incf	FSR,F			;to longitude min 10's
	movlw	"0"
	xorwf	INDF,W
	skpnz
	bsf	LM09_FLAG		;long min < 10
	movfw	INDF
	movwf	ascii_10s
	incf	FSR,F			;to longitude min 1's
	movfw	INDF
	movwf	ascii_1s
	call	ascii_to_hex
	movfw	hex_result
	movwf	lon_m0			;store as one hex byte
	incf	FSR,F			;skip decimal pt
	incf	FSR,F			;to longitude min 10ths
	movfw	INDF
	movwf	ascii_10s
	incf	FSR,F			;to longitude min 100ths
	movfw	INDF
	movwf	ascii_1s
	call	ascii_to_hex
	movfw	hex_result
	movwf	lon_m00			;store as one hex byte
	movfw	FSR
	movwf	offset
	call	findacomma		;go til comma seen
;
	incf	FSR,F			;to E W data
	movlw	"E"
	xorwf	INDF,W
	bnz	test_for_west
	bsf	EW_FLAG			;if E set flag
	goto	parse_speed
test_for_west
	movlw	"W"
	xorwf	INDF,W
	bnz	clean_house		;error, reset
	bcf	EW_FLAG
;
parse_speed
	movfw	FSR
	movwf	offset
	call	findacomma		;go til comma seen
	movlw	.2			;theres at least 1 digit
	addwf	FSR,F			;jump ahead 2, look for decimal pt
	movlw	"."
	xorwf	INDF,W
	bz	one_digit_speed		;saw it, 1 digit speed
	incf	FSR,F
	movlw	"."			
	xorwf	INDF,W
	bz	two_digit_speed		;saw it, 2 digit speed
;	
three_digit_speed
	movlw	.2			;no decimal seen, 3 digit speed
	subwf	FSR,F			;back 2 spaces to 100's
	movfw	INDF
	movwf	spd_100
	incf	FSR,F			;ahead to 10's
	movfw	INDF
	movwf	spd_10
	incf	FSR,F			;ahead to 1's
	movfw	INDF
	movwf	spd_1
	goto	trajectory
two_digit_speed
	movlw	.2			;saw decimal, 2 digit speed
	subwf	FSR,F			;back 2 spaces to 10's
	movfw	INDF
	movwf	spd_10
	incf	FSR,F
	movfw	INDF
	movwf	spd_1
	goto	trajectory		
one_digit_speed
	decf	FSR,F			;1 digit speed
	movfw	INDF
	movwf	spd_1	
;
trajectory
	incf	FSR,F
	movlw	"."			
	xorwf	INDF,W			;test for speed decimal
	bnz	clean_house		;error, reset
	movfw	FSR
	movwf	offset
	call	findacomma		;go til comma seen
;
parse_course
	movlw	.2			;theres at least 1 digit
	addwf	FSR,F			;jump ahead 2, look for decimal pt
	movlw	"."
	xorwf	INDF,W
	bz	one_digit_course	;saw it, 1 digit course
	incf	FSR,F
	movlw	"."			
	xorwf	INDF,W
	bz	two_digit_course	;saw it, 2 digit course	
three_digit_course
	movlw	.2			;no decimal seen, 3 digit course
	subwf	FSR,F			;back 2 spaces to 100's
	movfw	INDF
	movwf	crs_100
	incf	FSR,F			;ahead to 10's
	movfw	INDF
	movwf	crs_10
	incf	FSR,F			;ahead to 1's
	movfw	INDF
	movwf	crs_1
	goto	crunch_speed_course
two_digit_course
	movlw	.2			;2 digit course
	subwf	FSR,F			;back 2 spaces to 10's
	movfw	INDF
	movwf	crs_10
	incf	FSR,F
	movfw	INDF
	movwf	crs_1
	goto	crunch_speed_course		
one_digit_course
	decf	FSR,F			;1 digit coarse
	movfw	INDF
	movwf	crs_1	
;
crunch_speed_course
;
;	results in spd_10, crs_100, crs_1
;
	incf	FSR,F
	movlw	"."
	xorwf	INDF,W			;test for course decimal
	bnz	clean_house		;error, reset
;
	movfw	spd_100
	movwf	ascii_10s
	movfw	spd_10
	movwf	ascii_1s
	call	ascii_to_hex		;result in hex_result
;
	movfw	spd_100			;0-9 ascii
	sublw	"1"			;1 ascii	
	bnc	over_199knots
	movlw	.108			;
	goto	under_200knots
over_199knots
	movlw	.28			;
under_200knots
	addwf	hex_result,W
	movwf	spd_10			;result to spd_10
;
	movfw	spd_1
	movwf	ascii_10s
	movfw	crs_100
	movwf	ascii_1s
	call	ascii_to_hex
	movlw	.32			;
	addwf	hex_result,W
	movwf	crs_100			;result to crs_100
;
	movfw	crs_10
	movwf	ascii_10s
	movfw	crs_1
	movwf	ascii_1s
	call	ascii_to_hex
	movlw	.28			;
	addwf	hex_result,W
	movwf	crs_1			;result to crs_1
;
;	MSG bits B and C can be changed on the fly.
;	Off_duty, en_route, in_service and returning are valid.
;	The others including emergency cannot be set.
;	The 1st 3 bytes (lat_d10, lat_d1, lat_m10) contain this data.
;	PORTB is read, bits 6 and 7 determine (MSG B/C).
;	If the service bit is clear we add 0h to that byte.
;	If the service bit is set we add 20h to that byte.
;
crunch_latitude				
;
;	results in lat_d10, lat_d1, lat_m10, lat_m1, lat_m0, lat_m01
;
	movfw	PORTB			;read PORTB
	movwf	lookup_count		;temp store in lookup_count
;
	movlw	0x20			;bit A locked at 1
	addwf	lat_d10,F		;lat deg 10's
;
	movlw	0x20			;MSG bit B
	btfss	lookup_count,6		;test bit B
	movlw	0
	addwf	lat_d1,F		;lat deg 1's
;	
	movlw	0x20			;MSG bit C
	btfss	lookup_count,7		;test bit C
	movlw	0
	addwf	lat_m10,F		;lat min 10's
	clrf	lookup_count		;clear reg
;
	clrw				;clear W reg
	btfsc	NS_FLAG
	movlw	0x20			;flag set, North Latitude	
	addwf	lat_m1,F		;lat min 1's
;
	clrw				;clear W reg
	btfsc	LD100_FLAG		;test long >100
	goto	add_century
	btfsc	LD09_FLAG		;test long <10
add_century
	movlw	0x20			;flag set, +100
	addwf	lat_m0,F		;lat min 10ths
;
	clrw				;clear W reg
	btfss	EW_FLAG
	movlw	0x20			;flag clear, West longitude
	addwf	lat_m01,F		;lat min 100ths
;
crunch_longitude
;
;	results in 	lon_d, lon_m0, lon_m00
;	flags tested 	LD100_FLAG, LD09_FLAG, LM09_FLAG
;
crunch_longdeg
	btfsc	LD100_FLAG
	goto	morethan_100deg
	btfsc	LD09_FLAG
	goto	lessthan_10deg
between10and99deg
	movlw	.28			;no 100s stored, 10-99 and
	addwf	lon_d,F			;110-179 are treated alike
	goto	crunch_longmin		;imbedded LD100_FLAG points 
lessthan_10deg				;to actual value
	movlw	.118
	addwf	lon_d,F
	goto	crunch_longmin
morethan_100deg
	btfss	LD09_FLAG
	goto	between10and99deg	;between 110-179 deg
	movlw	.108			;between 100-109 deg
	addwf	lon_d,F
;
crunch_longmin
	btfsc	LM09_FLAG
	goto	lessthan_10min
	movlw	.28
	addwf	lon_m0,F
	goto	crunch_long100ths	
lessthan_10min
	movlw	.88
	addwf	lon_m0,F
;
crunch_long100ths
	movlw	.28
	addwf	lon_m00,F
;
;	Insert 6 latitude charactors to high memory
;	lat_d10, lat_d1, lat_m10, lat_m1, lat_m0, lat_m01
;	They replace the dummy "TO" field
;
insert_the_latitude	
	clrf	offset
	movfw	lat_d10	
	call	insert_to_high_memory
	movfw	lat_d1
	call	insert_to_high_memory
	movfw	lat_m10
	call	insert_to_high_memory
	movfw	lat_m1
	call	insert_to_high_memory
	movfw	lat_m0
	call	insert_to_high_memory
	movfw	lat_m01
	call	insert_to_high_memory
;
;	lon_d, lon_m0, lon_m00, spd_10, crs_100, crs_1
;
insert_the_longitude
        movlw   RAMM_START              ;reset data pointer
        movwf   FSR
;
	movfw	lon_d			;longitude degrees
	movwf	INDF
	incf	FSR,F
;
	movfw	lon_m0			;longitude minutes		
	movwf	INDF			;RAMM_START +1
	incf	FSR,F
;
	movfw	lon_m00			;longitude 1/100ths		
	movwf	INDF			;RAMM_START +2
	incf	FSR,F
;
;
	movfw	spd_10			;speed / course
	movwf	INDF			;RAMM_START +3
	incf	FSR,F
;
	movfw	crs_100
	movwf	INDF			;RAMM_START +4
	incf	FSR,F
;
	movfw	crs_1
	movwf	INDF			;RAMM_START +5
	incf	FSR,F
;
	movfw	m_symbol			
	movwf	INDF			;RAMM_START +6
	incf	FSR,F
;
	movlw	"/"
	movwf	INDF			;RAMM_START +7
	incf	FSR,F
;
	movlw	0			;end of data
	movwf	INDF
;
;	End of compressed sort routine
;============================================================
;
ready_to_tx
        clrf    INDF                    ;end of data marker (0)
	clrf	TMR1H			;26 mS LED flash
	clrf	TMR1L			;
	bcf	PIR1,TMR1IF
        bsf     LED                     ;LED on
;
	decf	buffer_1,F		;holdoff expired ?
	movfw	buffer_1
	bz	channel_check		;yes, check channel busy
;
	btfss	PIR1,TMR1IF
	goto	$-1			;
	goto	begin_parse		;restart parse, LED off
;
;------------------------------------------------------------
;	Looks for quiet on B:4 > 20.4 mS
;
channel_check
	call	look_busy
	btfsc	BUSY_FLAG
	goto	channel_check		;yes, wait
;
transmit_set_up
        clrf    TMR2                    ;
        bcf     PIR1,TMR2IF             ;clear flag
;
        movlw   b'10000000'             ;
        movwf   RCSTA                   ;serial rx off
	movlw	b'00000100'		;
	movwf	T2CON			;TMR2 on
;
        movlw   0xff
        movwf   AX25_crc_hi             ;reset CRC calc
        movwf   AX25_crc_lo
        bsf     PTT_OUT                 ;key radio
        movlw   0x28
        movwf   flag_count              ;40 flag delay
;
        movlw   high BIT_T
	movwf	TMR1H
	movlw	low BIT_T
	movwf	TMR1L
        bcf     PIR1,TMR1IF             ;clear flag
        call    tx_flags                ;send flags for tx
;
;------------------------------------------------------------
;	Path data is preconfigured and loaded from hi memory
;	Looks for (00) eof flag to end routine
;				
send_address_data
	movlw	BANK2_START
	movwf	FSR			;data address in hi memory
to_list_loop
	bsf	STATUS,IRP		;bank 2,3
	movfw	INDF			;read INDF at indirect offset
	bcf	STATUS,IRP		;bank 0,1
	bz	text_send		;saw eof flag (00f)
        call    reg_packet_out          ;send byte
        incf    FSR,F                   ;inc FSR
	goto	to_list_loop		
;------------------------------------------------------------
;					
text_send
        movlw   RAMM_START              ;reset data pointer
        movwf   FSR
	movlw	"`"
	btfss	mice_yn,0
	movlw	"$"			;mice_yn =0
	call	reg_packet_out		;send ` or $
text_send_loop
        movfw   INDF
        bz      tail_comment            ;zero, exit
        call    reg_packet_out          ;send byte
	movlw	RAMM_END
	xorwf	FSR,W
	bz	tail_comment		;overrun, finish up
        incf    FSR,F                   ;inc FSR
        goto    text_send_loop          ;get next
;
;------------------------------------------------------------
;	Tail comment sent every 8 transmits
;
tail_comment
	decfsz	count,F
	goto	finish_up
	movlw	.8
	movwf	count			;
	clrf	loop_count
comment_loop
	movfw	loop_count
	call	tail_text
	movwf	temp
	movfw	temp
	bz	finish_up
	call	reg_packet_out
	incf	loop_count,F
	goto	comment_loop
;------------------------------------------------------------
finish_up
        movlw   0xff
        xorwf   AX25_crc_hi,W
        movwf   temp                    ;inv, store hi byte
        movlw   0xff
        xorwf   AX25_crc_lo,W
        call    reg_packet_out
        movfw   temp
        call    reg_packet_out
        movlw   0x02
        movwf   flag_count
        call    tx_flags                ;send end flags
        btfss   PIR1,TMR1IF		;bit done?
        goto    $-1			;
        bcf     PTT_OUT                 ;un-key
;
	movlw	b'00000000'		;
	movwf	T2CON			;TMR2 off
;
	movlw	b'11001000'
        bsf     STATUS,RP0              ; bank 1
        movwf   VRCON                   ; set DAC midpoint
        bcf     STATUS,RP0              ; bank 0
	clrf	sine_offset		;
;
        movlw   b'10010000'             ;
        movwf   RCSTA                   ;serial rx on
;
        goto    clean_house             ;start over
;
;------------------------------------------------------------
;
;       Subroutines
;------------------------------------------------------------
;		Looks for 20.4 mS of quiet on B:4  
;		Sets flag if noise seen
look_busy
	bcf	BUSY_FLAG		;clear flag
	movlw	d'100'
	movwf	temp			;100 x 204 uS
	movfw	PORTB			;reset RBIF
	bcf	INTCON,RBIF		;clear flag
	clrf	TMR0			;timer 0 regs clear
look_busy_loop
	bcf	INTCON,T0IF		;timer 0 flag clear	
	btfss	INTCON,T0IF		;wait 204 uS
	goto	$-1			;
	decfsz	temp,F			;x 100
	goto	look_busy_loop
	btfsc	INTCON,RBIF		;B:4 toggle ?
	bsf	BUSY_FLAG		;yes, set flag
	return
;------------------------------------------------------------
;
;               Assumes byte is in W
reg_packet_out
        movwf   send_reg                ;W to send_reg
        movlw   0x08
        movwf   bit_count               ;8 bits
rotor_out
        rrf     send_reg,W              ;calc CRC
        call    crc_core
        rrf     send_reg,F              ;get bit to send
        skpc
        goto    zero_out
        incf    bit_stuff_count,F       ; b_s_c +1
        call    tx_one_bit              ;send a 1
        movfw   bit_stuff_count         ;check b_s_c
        xorlw   0x05
        skpnz                           ;b_s_f = 5 ?
        goto    zero_out                ;yes
count_the_bits
        decfsz  bit_count,F
        goto    rotor_out
        return
zero_out
        clrf    bit_stuff_count         ;reset b_s_f
        incf    next_freq,F
        call    tx_one_bit              ;send a 0
        goto    count_the_bits          ;
;
;------------------------------------------------------------
;               Sends (7e) b'01111110' flag_count times
tx_flags
        movlw   0x06
        movwf   bit_count               ;8 bits
        incf    next_freq,F
        call    tx_one_bit              ;send 0
flag_twist
        call    tx_one_bit              ;send 6 1's
        decfsz  bit_count,F
        goto    flag_twist
        incf    next_freq,F
        call    tx_one_bit              ;send 0
        decfsz  flag_count,F
        goto    tx_flags
        return                          ;end of flags
;
;------------------------------------------------------------
;       Assumes data bit is in Carry
crc_core
        movlw   1
        skpnc
        xorwf   AX25_crc_lo, F          ; crc calculation
        clrc
        rrf     AX25_crc_hi, F
        rrf     AX25_crc_lo, F
        skpc
        return
        movlw   H'08'
        xorwf   AX25_crc_lo, F
        movlw   H'84'
        xorwf   AX25_crc_hi, F
        return
;
;------------------------------------------------------------
;
tx_one_bit                              ;832 uS delay
        btfss   PIR1,TMR1IF             ;TMR1 timed out?
        goto    tx_one_bit              ;no
        movlw   high BIT_T              ;yes, reload
	movwf	TMR1H
	movlw	low BIT_T
	movwf	TMR1L
        bcf     PIR1,TMR1IF             ;clear flag
        btfss   next_freq,0             ;next_freq to W
        goto    lo_tone                 ;1, goto lo_tone
        bsf     TONE                    ;0, send 2200 Hz
        return
lo_tone
        bcf     TONE                    ;send 1200 Hz
        return
;------------------------------------------------------------
ee_write
	movlw	EEADR
	movwf	FSR
	movfw	ee_addr
	movwf	INDF
	movlw	EEDATA
	movwf	FSR
	movfw	ee_data
	movwf	INDF
;
	bsf	STATUS,RP0
	bsf	EECON1,WREN
off_int
	bcf	INTCON,GIE		;ints off
	btfsc	INTCON,GIE		;insure int are off
	goto	off_int				
	movlw	0x55
	movwf	EECON2
	movlw	0xaa
	movwf	EECON2
	bsf	EECON1,WR
ee_wr_wait
	btfsc	EECON1,WR
	goto	ee_wr_wait
	bcf	EECON1,WREN
	bsf	INTCON,GIE		;ints on
	bcf	STATUS,RP0
	return
ee_read
	movlw	EEADR
	movwf	FSR
	movfw	ee_addr
	movwf	INDF
	bsf	STATUS,RP0
	bsf	EECON1,RD
	bcf	STATUS,RP0
	movlw	EEDATA
	movwf	FSR
	movfw	INDF
	movwf	ee_data	
	return
;------------------------------------------------------------
;
findacomma
	movfw	offset
	movwf	FSR
	movlw	","
	xorwf	INDF,W
	skpnz
	return
	incf	offset,F
	goto	findacomma
;
;------------------------------------------------------------
;
ascii_to_hex
	clrf	hex_result
	movlw	0x30
	subwf	ascii_10s,F
	subwf	ascii_1s,F
	movfw	ascii_10s
	bz	low_nibble
hi_nibble_loop
	movlw	.10
	addwf	hex_result,F
	decfsz	ascii_10s,F
	goto	hi_nibble_loop
low_nibble
	movfw	ascii_1s
	addwf	hex_result,F
	return
;
;------------------------------------------------------------
;	Assumes data in W and offset correct
;
insert_to_high_memory
        movwf   temp                    ;copy to temp
        clrc                            ;clear carry
        rlf     temp,F                  ;shift left
	movlw	BANK2_START		;hi mem data start
	addwf	offset,W
	movwf	FSR			;reload correct FSR
        movfw   temp                    ;reload W
;
	bsf	STATUS,IRP		;bank 2,3
	movwf	INDF			;store W at indirect offset
	bcf	STATUS,IRP		;bank 0,1
	incf	offset,F
	return
;
;============================================================
;
	ORG 0x300
;
;	USER DATA ENTRY ROUTINE AREA
;------------------------------------------------------------
;	Interactive menu collects callsign(s), path(s), holdoff,
;	compress(y/n) and optional symbol.
;	Regs Used:
;	count, loop_count, next_freq, temp, rx_serial,
;	ee_addr, ee_data, FSR, INDF, tx_serial
;
;	Each callsign and optional SSID are processed as entered.
;	Data is stored in mid memory until routine end then
;	written to EEMEM.
;	Error resets and displays query again.
;	Data is echoed back to terminal, backspace edit enabled.
;------------------------------------------------------------
;
pwrup_msg				
        addwf   PCL,F
        dt      "FreeTrak V2.02 (c)2006",CR,LF
	dt	"Enter TOCALL",CR,LF,">",0
;------------------------------------------------------------
mycall_table
        addwf   PCL,F
        dt      CR,LF,"Enter MYCALL",CR,LF,">",0
;------------------------------------------------------------
via_1_table
	addwf	PCL,F
	dt	CR,LF,"Enter 1st VIA",CR,LF,">",0
;------------------------------------------------------------
via_2_table
        addwf   PCL,F
        dt      CR,LF,"Enter 2nd VIA",CR,LF,">",0
;------------------------------------------------------------
via_3_table
        addwf   PCL,F
        dt      CR,LF,"Enter 3rd VIA",CR,LF,">",0
;------------------------------------------------------------
txd_table
        addwf   PCL,F
        dt      CR,LF,"Enter HOLDOFF 1-99",CR,LF,">",0
;------------------------------------------------------------
mice_switch
        addwf   PCL,F
        dt      CR,LF,"Compressed Report Y/N",CR,LF,">",0
;------------------------------------------------------------
mice_symbol_table
        addwf   PCL,F
        dt      CR,LF,"Enter symbol code - <CR> for default car",CR,LF,">",0
;------------------------------------------------------------
	ORG 0x400
entry_done_table
        addwf   PCL,F
        dt      CR,LF,"Successful Configuration, adjust busy detect now",0
;------------------------------------------------------------
;
tocall_area
	movlw	MICE_SYMBOL +1
	movwf	loop_count
	movlw	RAMM_START		;clear midmemory store area
	movwf	FSR
tocall_area_cleanup
	clrf	INDF
	incf	FSR,F
	decfsz	loop_count,F
	goto	tocall_area_cleanup		
;
	bsf	STATUS,RP0		;bank 1
        movlw   b'00100000'		;
        movwf   TXSTA                   ;tx on, BRGH lo
	bcf	STATUS,RP0		;bank 0
	call	delay_26mS		;let things settle
	bsf	INTCON,GIE		;enable interrupts
tocall_restart
	clrf	FLAGS
        clrf    loop_count
get_tocall_loop
	movlw	high pwrup_msg
	movwf	PCLATH			;set up PCLATH
        movfw   loop_count
        call    pwrup_msg		;send power on msg
	movwf	next_freq
	xorlw	d'0'
	bz	get_tocall		;done
	movfw	next_freq
        movwf   tx_serial               ;load to send
	call	serial_tx
        incf    loop_count,F
        goto    get_tocall_loop
get_tocall
	call 	get_response
	btfsc	ERR_FLAG
	goto	tocall_restart		;error, start over
	btfsc	BT_FLAG
	goto	tocall_restart		;error, start over
	call	call_prep_store
	movlw	0x01
	movwf	offset			;save room for last byte loc
	call	xfer_to_midmem		;store call in midmem
;------------------------------------------------------------
mycall_area
	clrf	FLAGS
        clrf    loop_count
mycall_loop
        movfw   loop_count
        call    mycall_table		;send  mycall msg
	movwf	next_freq
	xorlw	d'0'
	bz	get_mycall		;done
	movfw	next_freq
        movwf   tx_serial               ;load to send
	call	serial_tx
        incf    loop_count,F
        goto    mycall_loop
get_mycall
	call 	get_response
	btfsc	ERR_FLAG
	goto	mycall_area		;error, start over
	btfsc	BT_FLAG
	goto	mycall_area
	call	call_prep_store
	movlw	0x08
	movwf	offset
	call	xfer_to_midmem		;store call in EEmem
;------------------------------------------------------------
via1_area
	bcf	ERR_FLAG
        clrf    loop_count
via1_loop
        movfw   loop_count
        call    via_1_table		;send via_1 msg
	movwf	next_freq
	xorlw	d'0'
	bz	get_via1		;done
	movfw	next_freq
        movwf   tx_serial               ;load to send
	call	serial_tx
        incf    loop_count,F
        goto    via1_loop
get_via1
	call 	get_response
	btfsc	BT_FLAG
	goto	holdoff_area		;no more calls
	btfsc	ERR_FLAG
	goto	via1_area		;error, start over
	call	call_prep_store	
	movlw	0x0f
	movwf	offset
	call	xfer_to_midmem		;store call in EEmem
;------------------------------------------------------------
via2_area
	bcf	ERR_FLAG
        clrf    loop_count
via2_loop
        movfw   loop_count
        call    via_2_table		;send via_2 msg
	movwf	next_freq
	xorlw	d'0'
	bz	get_via2		;done
	movfw	next_freq
        movwf   tx_serial               ;load to send
	call	serial_tx
        incf    loop_count,F
        goto    via2_loop
get_via2
	call 	get_response
	btfsc	BT_FLAG
	goto	holdoff_area		;no more calls
	btfsc	ERR_FLAG
	goto	via2_area		;error, start over
	call	call_prep_store
	movlw	0x16
	movwf	offset
	call	xfer_to_midmem		;store call in EEmem
;------------------------------------------------------------
via3_area
	bcf	ERR_FLAG
        clrf    loop_count
via3_loop
        movfw   loop_count
        call    via_3_table		;send via_3 msg
	movwf	next_freq
	xorlw	d'0'
	bz	get_via3		;done
	movfw	next_freq
        movwf   tx_serial               ;load to send
	call	serial_tx
        incf    loop_count,F
        goto    via3_loop
get_via3
	call 	get_response
	btfsc	BT_FLAG
	goto	holdoff_area		;no more calls
	btfsc	ERR_FLAG
	goto	via3_area		;error, start over
	call	call_prep_store
	movlw	0x1d
	movwf	offset
	call	xfer_to_midmem		;store call in EEmem
;------------------------------------------------------------
;		Gets holdoff count and stores to MIDMEM
;
holdoff_area
	clrf	FLAGS
        clrf    loop_count
holdoff_loop
        movfw   loop_count
        call    txd_table		;send holdoff msg
	movwf	next_freq
	xorlw	d'0'
	bz	get_holdoff		;done
	movfw	next_freq
        movwf   tx_serial               ;load to send
	call	serial_tx
        incf    loop_count,F
        goto    holdoff_loop
get_holdoff
	call 	get_response
	btfsc	ERR_FLAG
	goto	holdoff_area		;error, start over
	btfsc	BT_FLAG
	goto	holdoff_area		;no data entered
	clrf	temp
	movlw	RAML_START + 2
	xorwf	FSR,W
	bz	two_digit_ho
	movlw	RAML_START + 1
	xorwf	FSR,W
	bnz	holdoff_area		;more than 2 digit entry
	movlw	RAML_START		
	movwf	FSR
	movfw	INDF
	sublw	0x39
	bnc	holdoff_area		;INDF > 9 ascii
	movlw	0x30
	subwf	INDF,W
	bnc	holdoff_area		;INDF < 0 ascii
	movfw	INDF			;single digit entry
	andlw	0x0f
	movwf	temp
	goto	store_ho_midmem
;
two_digit_ho
	decf	FSR,F
	movfw	INDF
	sublw	0x39
	bnc	holdoff_area		;INDF > 9 ascii
	movlw	0x30
	subwf	INDF,W
	bnc	holdoff_area		;INDF < 0 ascii
	movfw	INDF			;single digit entry
	andlw	0x0f
	movwf	temp			;store 1's in temp
	movlw	RAML_START		
	movwf	FSR
	movfw	INDF			;10's digit
	sublw	0x39
	bnc	holdoff_area		;INDF > 9 ascii
	movlw	0x30
	subwf	INDF,W
	bnc	holdoff_area		;INDF < 0 ascii
	movfw	INDF			;single digit entry
	andlw	0x0f
	movwf	loop_count		;
	movlw	.10
	addwf	temp,F			;add 10 each loop
	decfsz	loop_count,F
	goto	$-2			;
;
store_ho_midmem
	btfsc	AH_FLAG
	goto	holdoff_area		;saw error, try again
	movlw	RAMM_START
	movwf	FSR
	movlw	HOLDOFF
	addwf 	FSR,F
	movfw	temp			;hex txd
	movwf	INDF			;store holdoff
;
;------------------------------------------------------------
;	checks which tx format - mic-e or RMC
;
compress_msg
	clrf	FLAGS
	clrf	loop_count
compress_msg_loop
        movfw   loop_count
        call    mice_switch		;send mic_e msg
	movwf	next_freq
	xorlw	d'0'
	bz	compress_msg_resp	;get response (Y/N)
	movfw	next_freq
        movwf   tx_serial               ;load to send
	call	serial_tx		;send
        incf    loop_count,F
        goto 	compress_msg_loop
compress_msg_resp
	call	get_response
	btfsc	ERR_FLAG
	goto	compress_msg		;error, start over
	btfsc	BT_FLAG
	goto	compress_msg		;no data, start over
	movlw	RAML_START
	movwf	FSR
	movfw	INDF
	movwf	temp			;store input in temp
;
store_mice_sw_reply
	movlw	RAMM_START
	movwf	FSR			;RAMM_START + MICE_SW
	movlw	MICE_SW
	addwf 	FSR,F
	movfw	temp			;
	movwf	INDF			;store in midmem
;
	movlw	RAMM_START
	movwf	FSR			;RAMM_START + MICE_SYMBOL
	movlw	MICE_SYMBOL
	addwf 	FSR,F
	movlw	">"			;default car symbol
	movwf	INDF			;store at offset
;
	movlw	"Y"
	xorwf	temp,W			;if compressed format
	bz	ask_mice_symbol		;ask for symbol type	
	movlw	"y"
	xorwf	temp,W
	bnz	mirror_to_eemem
;
ask_mice_symbol
	clrf	FLAGS
        clrf    loop_count
ask_mice_symbol_loop	
        movfw   loop_count
        call    mice_symbol_table	;send mic_e symbol msg
	movwf	next_freq
	xorlw	d'0'
	bz	get_mice_symbol		;
	movfw	next_freq
        movwf   tx_serial               ;load to send
	call	serial_tx		;send
        incf    loop_count,F
        goto	ask_mice_symbol_loop
;
get_mice_symbol
	call	get_response
	btfsc	ERR_FLAG
	goto	ask_mice_symbol		;error, start over
	btfsc	BT_FLAG
	goto	mirror_to_eemem		;blow thru, keep default
	movlw	RAML_START
	movwf	FSR
	movfw	INDF
	movwf	temp			;store input in temp
	movlw	RAMM_START
	movwf	FSR			;RAMM_START + MICE_SYMBOL
	movlw	MICE_SYMBOL
	addwf 	FSR,F
	movfw	temp			;get input data
	movwf	INDF			;store at offset
;
;------------------------------------------------------------
;
mirror_to_eemem
	clrf	FLAGS
	clrf	ee_addr
	clrf    offset
mirror_to_eemem_loop
	movlw	RAMM_START		;midram start
	addwf	offset,W		;add offset
	movwf	FSR
	movfw	INDF		
	movwf	ee_data			;xfer data to ee_data
	call	ee_write		;ee write
	incf	ee_addr,F		;inc ee_addr
	incf	offset,F		;inc offset
	movfw	offset
	sublw	MICE_SYMBOL		;test for last loc
	bc	mirror_to_eemem_loop	;loop back again
;		
;------------------------------------------------------------
;>>>>>>	entry_done_table crosses 256 byte boundry <<<<<<<
;
entry_done	
        clrf    loop_count
	movlw	high entry_done_table
	movwf	PCLATH
entry_done_loop
        movfw   loop_count
        call    entry_done_table	;send last msg
	movwf	next_freq
	xorlw	d'0'
	bz	busy_adjustment		;done, do busy adj
	movfw	next_freq
        movwf   tx_serial               ;load to send
	call	serial_tx		;send
        incf    loop_count,F
        goto    entry_done_loop
;
busy_adjustment				;noise detected = LED on
	call	look_busy		
	btfss	BUSY_FLAG		;
	goto	busyadj_LEDoff
	bsf	LED			;yes
	goto	busy_adjustment
busyadj_LEDoff	
	bcf	LED			;no
	goto	busy_adjustment		;loop forever...
;	
;------------------------------------------------------------
; 	Gets typed in reply to query and stores in 1st 9 bytes
;	of RAML. Checks for <CR>, blowthru (<CR> with no data)
;	and overrun.
;	2 flags - BT_FLAG (blowthru) and ERR_FLAG (error).	
;
get_response
	movlw	.9
	movwf	temp
	movlw	RAML_START + 9		;clear the temp storage
	movwf	FSR
clear_tempstor_loop
	clrf	INDF
	decf	FSR,F
	decfsz	temp,F
	goto	clear_tempstor_loop
response_loop
;
	btfss	NEW_DATA
	goto	response_loop		;wait for serial input
	bcf	NEW_DATA
;		
	movlw	CR			;
	xorwf	rx_serial,W		;is it a CR? 
	bz	response_end		;yes, next step
	movlw	BACKSPACE		;no, check for backspace
	xorwf	rx_serial,W		;backspace?
	bnz	store_this_entry	;no, do store routine
	movlw	RAML_START		;yes, check if at start
	xorwf	FSR,W
	bz	response_loop		;at start, do nothing
	decf	FSR,F			;prep to overwrite
	movlw	BACKSPACE
	movwf	tx_serial
	call	serial_tx		;backspace
	movlw	" "
	movwf	tx_serial
	call	serial_tx		;erase charactor
	movlw	BACKSPACE
	movwf	tx_serial		
	call	serial_tx		;backspace
	goto	response_loop
store_this_entry
	movfw	rx_serial
	movwf	INDF
	incf	FSR,F			;inc loc for next store
echo_back
	movfw	rx_serial
        movwf   tx_serial               ;load to send
	call	serial_tx
	goto	response_loop
response_end
	movlw	RAML_START
	xorwf	FSR,W			;blow thru?
	bz	blow_thru
	movfw	rx_serial
	movwf	INDF			;store <CR>
	movlw	RAML_START + 9		;CALLxx-Xx
	subwf	FSR,W
	bnc	$+2			;
	bsf	ERR_FLAG		;too long
	return
blow_thru
	bsf	BT_FLAG			;nothing entered
	return
;
;------------------------------------------------------------
;	Uses scratch pad RAM preloaded with 6 spaces and 30h 
;	SSID. Calls less than 6 chars are padded w/spaces.
;	1 or 2 decimal SSID is converted to 4 bit hex.
;	Each callsign entered is processed with this routine.	
;
call_prep_store
	movlw	.6
	movwf	offset
	movlw	SCRATCH_PAD
	movwf	FSR
prep_scratch_loop
	movlw	" "
	movwf	INDF			;6 spaces
	incf	FSR,F
	decfsz	offset,F
	goto	prep_scratch_loop
	movlw	b'00110000'		;set reserved bits
	movwf	INDF			;7th byte 0
;	
scan_call
	movlw	RAML_START
	addwf	offset,W
	movwf	FSR			;RAML_START + offset
	movlw	"-"
	xorwf	INDF,W			;is it a hyphen?
	bz	conv_call_ssid		;yes
	movlw	0x30			;no		
	subwf	INDF,W			;is it non-data?
	bnc	ssid_is_zero		;yes
	movlw	.6			;no, do we have 6 chars?
	xorwf	offset,W
	bz	ssid_is_zero		;yes, and no SSID
	movfw	INDF			;
	movwf	temp			;move data to scratchpad
	movlw	SCRATCH_PAD
	addwf	offset,W
	movwf	FSR			;store byte here
	movfw	temp
	movwf	INDF			;overwrite space
	incf	offset,F
	goto	scan_call		;get next
;
conv_call_ssid
	movlw	.2	
	addwf	FSR,F	
	movlw	0x30
	subwf	INDF,W			;test for <CR/LF>
	bnc	one_digit_ssid		;
	movfw	INDF	
	addlw	.10			;add 10 to ones
	movwf	temp
	goto	temp_store_ssid	
one_digit_ssid
	decf	FSR,F
	movfw	INDF
	movwf	temp			;store ssid
temp_store_ssid
	movlw	SCRATCH_PAD + 6
	movwf	FSR
	movfw	temp
	andlw	0x0f			;4 bit number
	iorlw	b'00110000'		;set reserved bits
	movwf	INDF			;overwrite zero	
;	
ssid_is_zero		
	return				;
;------------------------------------------------------------
;	Stores path and cfg data to be mirrored in EEMEM
;
xfer_to_midmem
	movlw	.7
	movwf	loop_count		;xfer 7 bytes
	clrf	count
xfer_to_midmem_loop
	movlw	SCRATCH_PAD
	addwf	count,W			;bytes 0-6 (7)
	movwf	FSR			;lo mem indirect data loc
	movfw	INDF			;get scratchpad data
	movwf	flag_count		;store in flag_count
	movlw	RAMM_START		;mid mem start point
	addwf	offset,W		;add to offset
	movwf	FSR			;mid mem indirect data loc
	movfw	flag_count		;get stored byte
	movwf	INDF			;write data to indirect reg
	incf	count,F			;inc pointers
	incf	offset,F
	decfsz	loop_count,F		;dec loop counter
	goto	xfer_to_midmem_loop	;loop back
	movlw	1			;calculate end point so far
	subwf	offset,F		;
	movlw	RAMM_START		;	
	movwf	FSR			;
	movfw	offset			;		
	movwf	INDF			;store last byte loc at 0
	return
;
;------------------------------------------------------------
;	Transfers EE call/path data to bank 2 hi memory 
;	Data is preconfigured (left shifted) for transmit
;	1st EE byte is eof address
;	0x00 is eof in hi memory
;
load_stored_path
	clrf	loop_count		;offset
	clrf	ee_addr			;reset eeaddress	
	call	ee_read			;get eedata
	movfw	ee_data
        movwf	count			;1st byte is eof loc
	sublw	.13
	bc	no_path_stored		;invalid 
	movfw	ee_data
	sublw	.35
	bnc	no_path_stored		;invalid
	incf	ee_addr,F		;point to second byte
stored_path_loop
	call	ee_read			;get eedata
	movfw	ee_data
        movwf   temp                    ;copy to temp
        clrc                            ;clear carry
        rlf     temp,F                  ;shift left
	movlw	BANK2_START		;hi mem data start
	addwf	loop_count,W
	movwf	FSR			;reload correct FSR
        movfw   temp                    ;reload W
;
	bsf	STATUS,IRP		;bank 2,3
	movwf	INDF			;store W at indirect offset
	bcf	STATUS,IRP		;bank 0,1
;
	incf	loop_count,F
	incf	ee_addr,F
	movfw	count
	xorwf	ee_addr,W
	bz	end_of_path
        goto    stored_path_loop        ;get next
;
;------------------------------------------------------------
end_of_path
	call	ee_read			;get eedata
	movfw	ee_data
        movwf   temp                    ;copy to temp					
        setc                            ;set carry
        rlf     temp,F                  ;shift left
	movlw	BANK2_START		;reload correct FSR
	addwf	loop_count,W
	movwf	FSR
        movfw   temp                    ;reload W (last SSID)
;
	bsf	STATUS,IRP		;bank 2,3
	movwf	INDF			;store W at indirect offset
	bcf	STATUS,IRP		;bank 0,1
;
	incf	FSR,F
        movlw	0x03                    ;0x03
	bsf	STATUS,IRP		;bank 2,3
	movwf	INDF			;store W at indirect offset
	bcf	STATUS,IRP		;bank 0,1
	incf	FSR,F
        movlw	0xf0                    ;0xf0
	bsf	STATUS,IRP		;bank 2,3
	movwf	INDF			;store W at indirect offset
	bcf	STATUS,IRP		;bank 0,1
	incf	FSR,F
        movlw	0	                ;0 - eof flag
	bsf	STATUS,IRP		;bank 2,3
	movwf	INDF			;store W at indirect offset
	bcf	STATUS,IRP		;bank 0,1
;
transmit_delay_read
	movlw	HOLDOFF			;txd loc in eemem
	movwf	ee_addr
	call	ee_read			;get eedata
	movfw	ee_data
	movwf	beacon_count		;holdoff loaded
mice_switch_read
	movlw	MICE_SW			;mice sw loc in eemem
	movwf	ee_addr
	call	ee_read			;get eedata
	movfw	ee_data
	movwf	mice_yn			;store
	movlw	"Y"
	xorwf	mice_yn,W		;send format stnd/comp?
	bz	itsa_yes
	movlw	"y"			
	xorwf	mice_yn,W		;check lower case
	bz	itsa_yes
	clrf	mice_yn			;0 = standard msg format
	return
itsa_yes
	movlw	1
	movwf	mice_yn			;1 = compress msg format
;
mice_symbol_read
	movlw	MICE_SYMBOL		;mice symbol loc in eemem
	movwf	ee_addr
	call	ee_read			;get eedata
	movfw	ee_data
	movwf	m_symbol		;stored in m_symbol
	return	
;
;------------------------------------------------------------
no_path_stored
	bsf	LED
	call	delay_26mS
	bcf	LED
	call	delay_26mS
	goto	no_path_stored
;
;------------------------------------------------------------
serial_tx
        btfss   PIR1,TXIF
        goto    serial_tx
	nop
	nop
	movfw	tx_serial
        movwf   TXREG                   ;load to send
	return
;
;------------------------------------------------------------
;	sinewave lookup table
;
	ORG	0x700
;
sine_table
        addwf   PCL,F                   ;add offset to pc
;
	dt	0x05,0x08,0x0a,0x0d,0x0e,0x0f,0x0e,0x0d
	dt	0x0a,0x08,0x05,0x02,0x01,0x00,0x01,0x02 
;------------------------------------------------------------
int_service
        movwf   temp_W			;(1) [8]
        swapf   STATUS,W		;(1)
	bcf	STATUS,RP0		;(1) insure bank 0
        movwf   temp_S			;(1)
	movfw	PCLATH			;(1)
	movwf	temp_PCLH		;(1)	
        btfss   PIR1,TMR2IF		;(1/2) TMR2IF flag set?
        goto    serial_receive_handler	;(2) no, check serial rx
;
;	avg 45 clock cycles
tone_job
        bcf     PIR1,TMR2IF             ;(1) [4] clear flag
        btfsc   TONE			;(1/2)
        goto    tone_1			;(2)
tone_0
	movlw	LO_TONE			;(1)[6]
	bsf	STATUS,RP0		;(1)bank 1
	movwf	PR2			;(1)timer 2 1200 Hz
        bcf     STATUS,RP0              ;(1)bank 0
        goto	set_dac			;(2)
;
tone_1
	movlw	HI_TONE			;(1)[4]
	bsf	STATUS,RP0		;(1)bank 1
	movwf	PR2			;(1)timer 2 2200 Hz
        bcf     STATUS,RP0              ;(1)bank 0
;
set_dac
	incf	sine_offset,F		;(1)[20]next step
	movlw	HIGH sine_table		;(1)
	movwf	PCLATH			;(1)load for lookup
	movlw	b'00001111'		;(1)
	andwf	sine_offset,W		;(1)
        call    sine_table		;(5)
        movwf   phase                   ;(1) store results
	movlw	HIGH set_dac		;(1)
	movwf	PCLATH			;(1)restore PCLATH
        movlw   b'11000000'		;(1)	
        iorwf   phase,W			;(1)
        bsf     STATUS,RP0              ;(1) bank 1
        movwf   VRCON                   ;(1) set DAC
        bcf     STATUS,RP0              ;(1) bank 0
        goto    outta_here		;(2)
;
serial_receive_handler
	btfss	PIR1,RCIF
	goto	outta_here
	movlw	1
	btfss	TOGGLE
	movwf	buffer_1		;set up forced send if A:4 low	
	bcf	NEW_DATA		;clear flag
        movlw   b'00000110'
        andwf   RCSTA,W                 ;check error flags
        bnz     serial_error
        movfw   RCREG
        movwf   rx_serial               ;xfer data byte to rx_serial
	bsf	NEW_DATA		;yes, byte rx'd
        goto    outta_here
serial_error
	bsf	START_SERIAL		;set serial routine to beginning
        btfss   RCSTA,FERR
        goto    check_orrun
        movfw   RCREG                   ;read byte, don't store
check_orrun
        btfss   RCSTA,OERR
        goto    outta_here
        bcf     RCSTA,CREN
        nop
        bsf     RCSTA,CREN              ;cycle CREN bit to reset
outta_here				
	movfw	temp_PCLH		;(1)[8]
	movwf	PCLATH			;(1)
        swapf   temp_S,W		;(1)
        movwf   STATUS			;(1)
        swapf   temp_W,F		;(1)
        swapf   temp_W,W		;(1)
        retfie                          ;(2) return to program
;------------------------------------------------------------
delay_26mS
	clrf	TMR1L
	clrf	TMR1H
	bcf	PIR1,TMR1IF
delay_26mS_loop
	nop
	btfss	PIR1,TMR1IF		;26 mS delay
	goto	delay_26mS_loop
	return	
;------------------------------------------------------------
        end
;------------------------------------------------------------
;
;Copyright (c) 2007, Mike Berg
;All rights reserved.

;Redistribution and use in source and binary forms, with or without 
;modification, are permitted provided that the following conditions 
;are met:

;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;Redistributions in binary form must reproduce the above copyright 
;notice, this list of conditions and the following disclaimer in the 
;documentation and/or other materials provided with the distribution. 
;Neither the name of the <ringolake.com> nor the names of its 
;contributors may be used to endorse or promote products derived from 
;this software without specific prior written permission. 
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
;"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
;LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
;FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
;COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
;INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
;BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
;OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
;AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
;OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
;THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
;DAMAGE.
